import xbmcaddon
import datetime
import time
from lib.logging import *
from lib.oled import *
from lib.charset import *
from lib.settings import *
from lib.player import *


class Monitor(xbmc.Monitor):
    def __init__(self):
        super(Monitor, self).__init__()
        self._settingsChangedCallback = None

    def setSettingsChangedCallback(self, callback):
        self._settingsChangedCallback = callback

    def onSettingsChanged(self):
        if (self._settingsChangedCallback is not None):
            self._settingsChangedCallback.onSettingsChanged()


class OledAddon():
    def __init__(self, monitor):
        self._player = XBMCPlayer()
        self._player.setOnAVStartedCallback(self)
        self._player.setOnPlayBackStoppedCallback(self)
        self._player.setOnPlayBackEndedCallback(self)
        self._monitor = monitor
        self._monitor.setSettingsChangedCallback(self)
        self._settings = OledSettings()
        self._settings.readSettings()
        self._status = "idle"
        self._wait = 0
        self.testx = 0
        self._oled = Oled(self._settings.i2cAddress(),
                          self._settings.displayType(),
                          self._settings.flipDisplay())
        self._oled.setBrightness(self._settings.brightness())

    def __del__(self):
        self._oled.close()

    def onAVStarted(self):
        self._oled.clear()
        self._oled.setBrightness(self._settings.playbackBrightness())
        if (xbmc.Player().isPlayingVideo()):
            self._status = "video"
        elif (xbmc.Player().isPlayingAudio()):
            self._status = "audio"

    def onPlayBackEnded(self):
        self._oled.clear()
        self._status = "waiting"
        self._wait = 0

    def onPlayBackStopped(self):
        self._oled.clear()
        self._status = "waiting"
        self._wait = 0

    def onSettingsChanged(self):
        self._settings.readSettings()
        self._oled = Oled(self._settings.i2cAddress(),
                          self._settings.displayType(),
                          self._settings.flipDisplay())
        self._oled.setBrightness(self._settings.brightness())

    def run(self):
        while not self._monitor.abortRequested():
            if self._monitor.waitForAbort(0.5):
                break
            self.update()
        self._oled.close()

    def update(self):
        # Fix for Kodi's broken onPlayBackStopped
        # Wait a couple of seconds to make sure playback has stopped
        # and we havent just changed track
        if self._status == "waiting":
            self._wait += 1
            if self._wait >= 4:
                self._oled.setBrightness(self._settings.brightness())
                self._status = "idle"
        ############################################################

        if self._status == "idle":
            now = datetime.datetime.today()
            seconds = (now.hour * 60) + now.minute
            self._oled.drawTime(
                seconds, 0, 0, sevenSeg, False, True)
            self._oled.display()

        if self._status == "video":
            try:
                playtime = int(xbmc.Player().getTime())
                totalTime = int(xbmc.Player().getTotalTime())
                if (playtime >= 0):
                    self._oled.drawProgress(playtime, totalTime)
                    self._oled.drawTime(playtime, 0, 0, sevenSeg, True, True)
                    self._oled.display()
            except:
                pass

        if self._status == "audio":
            try:
                playtime = int(xbmc.Player().getTime())
                totalTime = int(xbmc.Player().getTotalTime())
                if (playtime >= 0):
                    self._oled.drawProgress(playtime, totalTime)
                    self._oled.drawTime(
                        playtime, 0, 0, sevenSeg, False, True)
                    self._oled.display()
            except:
                pass


if __name__ == "__main__":
    logNotice('Service started')
    monitor = Monitor()
    oled = OledAddon(monitor)
    oled.run()
    logNotice('Service stopped')
